<?php

/**
 * @file
 * Core functionality for CiviCRM Taxonomy-Groups.
 */


/**
 * Find CiviCRM Group ID for a given Vocabulary/Term.
 */
function civicrm_taxonomy_get($vid, $tid = NULL, $reset = FALSE) {
  static $index = array();
  if (isset($index[$vid][$tid]) && !$reset) {
    return $index[$vid][$tid];
  }

  $sql = "SELECT ctg.ctgid, ctg.vid, ctg.tid, ctg.gid, ctg_p.gid AS `parent_gid` FROM {civicrm_taxonomy} ctg 
LEFT JOIN {term_hierarchy} th USING (tid)
LEFT JOIN {civicrm_taxonomy} ctg_p ON th.parent = ctg_p.tid
WHERE ctg.vid = %d AND ctg.tid = %d;";
  $res = db_query($sql, $vid, $tid);

  // TODO: Could be made more efficient - 
  // If just $vid passed in, why not grab the whole vocabulary and cache?
  $row = db_fetch_array($res);
  
  $index[$vid][$tid] = $row;
  return $row;
}


/**
 * Remove a CiviCRM Taxonomy-Group.
 */
function civicrm_taxonomy_link_delete($vid, $tid, $purge_group = FALSE) {
  $settings = variable_get('civicrm_taxonomy_settings', array());
  $link = civicrm_taxonomy_get($vid, $tid);

  if ($purge_group) {
    civicrm_taxonomy_group_delete($link['gid']);
  }
  
  db_query('DELETE FROM {civicrm_taxonomy} WHERE vid = %d AND tid = %d',
           array($vid, $tid));
}


/** 
 * Create/Update a CiviCRM Taxonomy-Group.
 */
function civicrm_taxonomy_link_write($term) {
  // See if link already exists
  if (is_array($term)) {
    $term = (object) $term;
  }
  $link = civicrm_taxonomy_get($term->vid, $term->tid);
  $gid = NULL;
  if ($link['gid']) {
    $gid = $link['gid'];
  }
  
  $parent = isset($term->parents) ? array_pop($term->parents) : NULL;
  $parent_link = civicrm_taxonomy_get($term->vid, $parent);
  $pid = $parent_link['gid'];

  $gid = civicrm_taxonomy_group_write($term->name, $term->description, $pid, $gid);
 
  if (!$gid) {
    drupal_set_message('There was an error; A CiviCRM Group could not be created for '. $term->name .'.', 'error');
    return FALSE;
  }

  if (!isset($link['gid'])) {
    $record = array('vid' => $term->vid, 'tid' => $term->tid, 'gid' => $gid);
    drupal_write_record('civicrm_taxonomy', $record);
  }
  return TRUE;
}


/**
 * Create/Update a CiviCRM Group and return its ID. (Wrapper for CiviCRM API.)
 */
function civicrm_taxonomy_group_write($name, $desc, $parent_id = NULL, $group_id = NULL) {
  if (!civicrm_initialize()) {
    return;
  }
    
  require_once drupal_get_path('module', 'civicrm') .'/../api/v2/Group.php';
  $settings = variable_get('civicrm_taxonomy_settings', array());

  $name = $settings['misc']['prefix'] .' '. $name;
  $name = civicrm_taxonomy_name_validate($name, $group_id);
  $params = array(
    'name' => $name,
    'title' => $name,
    'description' => $desc,
    'is_active' => 1,
    'visibility' => 'User and User Admin Only',
    'group_type' => array('2' => 1),
  );
  if ($group_id) {
    $params['id'] = $group_id;
  }
  if ($parent_id) {
    $params['parents'] = $parent_id;
  }

  $result = &civicrm_group_add($params);
  if (civicrm_error($result)) {
    drupal_set_message($result['error_message'], 'error');
  }
  else {
    return $result['result'];
  }
}


/**
 * Find the best name match that isn't already taken by a group.
 */
function civicrm_taxonomy_name_validate($name, $group_id = NULL) {
  if (!civicrm_initialize()) {
    return;
  }  
  require_once drupal_get_path('module', 'civicrm') .'/../api/v2/Group.php';

  $new_name = $name;
  $params = array('name' => $new_name);
  $blocked = TRUE;
  $i = 1;

  $results = civicrm_group_get($params);
  foreach (array_keys($results) as $key) {
    if ($results[$key]['id'] == $group_id) {
      unset($results[$key]);
    }
  }

  unset($results['error_message']);
  $blocked = count($results);
  
  while ($blocked) {
    $new_name = $name.' '. $i;
    $params = array('name' => $new_name);
    $i++;

    $results = civicrm_group_get($params);
    foreach (array_keys($results) as $key) {
      if ($results[$key]['id'] == $group_id) {
        unset($results[$key]);
      }
    }

    unset($results['error_message']);

    $blocked = count($results);
  }


  return $new_name;
}


/**
 * Delete a CiviCRM Group. (Wrapper for CiviCRM API.)
 */
function civicrm_taxonomy_group_delete($group_id) {
  // Call civicrm API here.
  //_dbg(array('Deleting: ', $group_id));

  if (!civicrm_initialize()) {
    return;
  }
    
  require_once drupal_get_path('module', 'civicrm') .'/../api/v2/Group.php';

  $params = array('id' => $group_id);
  $result = &civicrm_group_delete($params);

  if (civicrm_error($result)) {
    drupal_set_message($result['error_message'], 'error');
  }
  else {
    return TRUE;
  }
}

/**
 * Retrieve all terms associated with a user.
 */
function civicrm_taxonomy_user_get_terms($uid) {
  // TODO: Implement this.
  if (!civicrm_initialize()) {
    return array();
  }

  require_once(drupal_get_path('module', 'civicrm') .'/../api/v2/Group.php');
  require_once(drupal_get_path('module', 'civicrm') .'/../api/v2/GroupContact.php');
  require_once(drupal_get_path('module', 'civicrm') .'/../api/v2/UFGroup.php');

  $contact_ID = civicrm_uf_match_id_get($uid);

  $params = array(
    'contact_id' => $contact_ID,
  );
  $res = civicrm_group_contact_get($params);

  if (civicrm_error($result)) {
    return $result['error_message'];
  }
  else {
    $gids = array();
    //_dbg($res);
    foreach ($res as $gid => $group) {
      $gids[] = $group['group_id'];
    }

    $tids = civicrm_taxonomy_tids_gids($gids, 'gid', 'tid');
    return $tids;
  }
}

/**
 * Set a user all terms for a vocabulary associated with a user.
 */
function civicrm_taxonomy_user_set_terms($uid, $vid, $tids, $mode = 'uid') {

  // TODO: Implement this.
  if (!civicrm_initialize()) {
    return array();
  }

  require_once(drupal_get_path('module', 'civicrm') .'/../api/v2/Group.php');
  require_once(drupal_get_path('module', 'civicrm') .'/../api/v2/GroupContact.php');
  require_once(drupal_get_path('module', 'civicrm') .'/../api/v2/UFGroup.php');

  if ($mode == 'uid') {
    $contact_id = civicrm_uf_match_id_get($uid);
  }
  else {
    $contact_id = $uid;
  }

  //_dbg(array($uid, $vid, $tids, $mode), 'mail');
  if (!$contact_id) {
    watchdog('civicrm_taxonomy', 'Attempted to add terms to undefined user.');
    return;
  }

  // First, retrieve current groups
  $params = array(
    'contact_id' => $contact_id,
  );
  $res = civicrm_group_contact_get($params);

  if (civicrm_error($result)) {
    return $result['error_message'];
  }
  else {
    $gids = array();
    //_dbg($res);
    foreach ($res as $gid => $group) {
      $gids[] = $group['group_id'];
    }
  }

  // Next, retrieve tid-gid links for the specified vocabulary
  $sql = "SELECT vid, tid, gid FROM {civicrm_taxonomy} WHERE vid = %d";
  $res = db_query($sql, $vid);

  $add = array();
  $del = array();
  while ($row = db_fetch_array($res)) {
    if (in_array($row['gid'], $gids) && !in_array($row['tid'], $tids)) {
      $del[] = $row['gid'];
    }
    if (!in_array($row['gid'], $gids) && in_array($row['tid'], $tids)) {
      $add[] = $row['gid'];
    }
  }

  //_dbg(array($add, $del), 'mail');

  // Process add and del lists.
  foreach ($add as $group_id) {
    $params = array(
      'contact_id' => $contact_id, 
      'group_id' => $group_id,
    );
    civicrm_group_contact_add($params);
  }
  foreach ($del as $group_id) {
    $params = array(
      'contact_id' => $contact_id, 
      'group_id' => $group_id,
    );
    civicrm_group_contact_remove($params);
  }
}

/**
 * Convert taxonomy terms to CiviCRM groups, or vice versa.
 */
function civicrm_taxonomy_tids_gids($input, $from = 'tid', $to = 'gid') {
  if (!in_array($from, array('tid', 'gid')) ||
      !in_array($to, array('tid', 'gid'))) {
    return;
  }
  if (count($input)) {
    $res = db_query("SELECT $to FROM {civicrm_taxonomy} WHERE $from IN (".
                    implode(',', $input) .')');
    $output = array();
    while ($row = db_fetch_array($res)) {
      $output[] = $row[$to];
    }
    return $output;
  }
  else {
    return array();
  }
}


