<?php

/**
 * @file
 * Settings page for CiviCRM Taxonomy-Groups.
 **/

module_load_include('inc', 'civicrm_taxonomy', 'core');

/**
 * Form builder for settings page.
 */
function civicrm_taxonomy_settings($form_state) {
  $settings = variable_get('civicrm_taxonomy_settings', array());
  $form = array();
  $form['civicrm_taxonomy_settings'] = array(
    '#tree' => TRUE,
  );

  $vocabs = taxonomy_get_vocabularies();
  foreach ($vocabs as $vid => $voc) {
    $options[$vid] = $voc->name;
  }
  $form['civicrm_taxonomy_settings']['vocabs'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Vocabularies'),
    '#description' => t('Select which vocabularies should be mirrored in CiviCRM as groups.'),
    '#options' => $options,
    '#default_value' => $settings['vocabs'],
  );

  $form['civicrm_taxonomy_settings']['delete'] = array(
    '#type' => 'fieldset',
    '#title' => t('Deletion settings'),
    '#description' => t('The settings below apply only to the selected vocabularies above. Selecting "no" may result in orphaned or duplicate groups. Selecting "yes" may result in unintentional data loss.'),
    '#tree' => TRUE,
  );

  $form['civicrm_taxonomy_settings']['delete']['taxonomy'] = array(
    '#title' => t('Vocabulary/Term deletion'),
    '#type' => 'radios',
    '#description' => t('When a mirrored taxonomy vocabulary or term is deleted, should the corresponding CiviCRM group also be deleted?'),
    '#options' => array(0 => t('No'), 1 => t('Yes'),),
    '#default_value' => $settings['delete']['taxonomy'],
    '#required' => TRUE,
  );

  $form['civicrm_taxonomy_settings']['delete']['disable'] = array(
    '#title' => t('Disable deletion'),
    '#type' => 'radios',
    '#description' => t('When a vocabulary mirroring is disabled from this settings page, should the corresponding CiviCRM groups also be deleted? Note that changing this setting will not go into effect until after any changes to the vocabulary settings above have been made.'),
    '#options' => array(0 => t('No'), 1 => t('Yes')),
    '#default_value' => $settings['delete']['disable'],
    '#required' => TRUE,
  );

  $form['civicrm_taxonomy_settings']['delete']['uninstall'] = array(
    '#title' => t('Uninstall deletion'),
    '#type' => 'radios',
    '#description' => t('When this module is uninstalled, should all CiviCRM taxonomy-groups be deleted?'),
    '#options' => array(0 => t('No'), 1 => t('Yes'),),
    '#default_value' => $settings['delete']['uninstall'],
    '#required' => TRUE,
  );

  $form['civicrm_taxonomy_settings']['integration'] = array(
    '#type' => 'fieldset',
    '#title' => t('Integration settings'),
    '#tree' => TRUE,
  );

  $form['civicrm_taxonomy_settings']['integration']['user'] = array(
    '#title' => t('Enable group subscriptions page in user account section.'),
    '#description' => t('Note that CiviCRM also provides similar funcationality directly. This is probably not necessary for everyone.'),
    '#type' => 'radios',
    '#options' => array(0 => t('No'), 1 => t('Yes')),
    '#default_value' => $settings['integration']['user'],
  );
    
  $form['civicrm_taxonomy_settings']['integration']['register'] = array(
    '#title' => t('Enable group subscriptions on user registration page.'),
    '#description' => t('Note that CiviCRM also provides similar funcationality directly. This is probably not necessary for everyone.'),
    '#type' => 'radios',
    '#options' => array(0 => t('No'), 1 => t('Yes')),
    '#default_value' => $settings['integration']['register'],
  );

  $form['civicrm_taxonomy_settings']['misc']['prefix'] = array(
    '#title' => t('CiviCRM Group prefix'),
    '#description' => t('Prefix to append to group titles in CiviCRM. This can help to prevent conflicts, especially with Smart Groups.'),
    '#type' => 'textfield',
    '#default_value' => $settings['misc']['prefix'],
  );
    
  $form['#submit'][] = 'civicrm_taxonomy_settings_submit';

  return system_settings_form($form);
}


/**
 * Submit handler for settings form.
 */
function civicrm_taxonomy_settings_submit($form, &$form_state) {
  $old_settings = variable_get('civicrm_taxonomy_settings', array());

  // Find new vocabularies & create groups
  foreach ($form_state['values']['civicrm_taxonomy_settings']['vocabs'] as $vid => $enabled) {
    if ($enabled && !$old_settings['vocabs'][$vid]) {
      civicrm_taxonomy_enable($vid);
    }
  }

  // Find disabled vocabularies & remove
  foreach ($old_settings['vocabs'] as $vid => $enabled) {
    if ($enabled && !$form_state['values'][$vid]) {
      civicrm_taxonomy_disable($vid);
    }
  }
}


/**
 * Disable a CiviCRM hierarchy based on a vocabulary.
 */
function civicrm_taxonomy_disable($vid) {
  watchdog('civicrm_taxonomy', 'Deleted vocabulary !vid from CiviCRM Taxonomy-Groups.', array('!vid' => $vid));
  $settings = variable_get('civicrm_taxonomy_settings', array());

  $sql = 'SELECT * FROM {civicrm_taxonomy} WHERE vid = %d';
  $res = db_query($sql, $vid);

  while ($row = db_fetch_array($res)) {
    civicrm_taxonomy_link_delete($row['vid'], $row['tid'], $settings['delete']['disable']);
  }
}


/**
 * Create or enable a new a CiviCRM hierarchy based on a vocabulary.
 */
function civicrm_taxonomy_enable($vid) {
  // Was a link created before? Use if found; otherwise create a new group.
  $ctg = civicrm_taxonomy_get($vid);
  //_dbg($ctg);
  $voc = taxonomy_vocabulary_load($vid);
  if ($ctg['vid']) {
    // TODO: Verify group's existence?
    $gid = $ctg['gid'];
    $name = civicrm_taxonomy_name_validate($voc->name);
    civicrm_taxonomy_group_write($name, $voc->description, NULL, $gid);
  }
  else {
    $gid = civicrm_taxonomy_group_write($voc->name, $voc->description, NULL);
    $record = array('vid' => $vid, 'tid' => $tid, 'gid' => $gid);
    drupal_write_record('civicrm_taxonomy', $record);
  }

  // For each term in this vocabulary, create a CiviCRM Group if necessary
  civicrm_taxonomy_enable_tree($vid);
}


/**
 * Recursive helper function.
 */
function civicrm_taxonomy_enable_tree($vid, $parent = 0) {
  $tree = taxonomy_get_tree($vid, $parent, -1, 1);
  $parent_link = civicrm_taxonomy_get($vid, $parent, TRUE);
  $results = array();
  foreach ($tree as $term) {
    civicrm_taxonomy_link_write($term);
    // Once a record is written, recurse and update children
    civicrm_taxonomy_enable_tree($vid, $term->tid);
  }
}
